#######################################
# The structure of the file: 
#   1. helper functions
#   2. pseudo-dataset creation
#   3. commented example of usage
#
#######################################
# 1. helper functions
#######################################
library(phangorn)

norm.hamming <- function (a, b){
	a <- as.character(a)
    b <- as.character(b)
    if (length(a) == 0 || length(b) == 0)
        return(numeric(0))
    if (length(a) != length(b))
        warning("Unequal length!")
    diff_count = 0
    skip_count = 0
    for (i in 1:length(a))
    {
    	if (a[i] == "?" || b[i] == "?")
    		skip_count <- skip_count + 1
    	else
    		if (a[i] != b[i])
    			diff_count <- diff_count + 1
    }
    return(diff_count/(length(a) - skip_count))
}

norm.hamming.matrix <- function(data){
	n <- length(data)
	res <- matrix(rep(0, times=n^2), n, n)
	for (i in 1:n){
		for (j in 1:n){
			if (i > j)
				res[i,j] <- norm.hamming(data[[i]], data[[j]])
				res[j,i] <- res[i,j]
		}
	}
	return(res)
}

NJ_from_cognacy_data <- function(data){
	dist.matrix <- norm.hamming.matrix(data)
	dimnames(dist.matrix) <- list(names(data), names(data))
	NJ(dist.matrix)
}


#######################################
# 2. pseudo-dataset creation
#######################################

dict.omission.create.pseudo.dataset <- function(original.data, p.qu.to.1, p.0.to.1) {
	pseudo.data <- original.data
	
	for(j in 1:length(original.data))
		for(k in 1:length(original.data[[j]]))
		{
			if(original.data[[j]][k] == "0")
				if(runif(1, 0, 1) <= p.0.to.1)
					pseudo.data[[j]][k] <- "1"
			if(original.data[[j]][k] == "?")
				if(runif(1, 0, 1) <= p.qu.to.1)
					pseudo.data[[j]][k] <- "1"
				else 
					pseudo.data[[j]][k] <- "0"
		}
	pseudo.data
}


#######################################
# 3. commented example of usage
#######################################

#data <- read.nexus.data("Lezgian-recoded.nex")
#data.NJ <- NJ_from_cognacy_data(data)

#n_pseudo_copies = 1000
#p.qu.to.1 = 0.13
#p.0.to.1 = 0.03

#dict.omission.pseudodatasets.NJ <- lapply(1:n_pseudo_copies, function(i) {
#	if (i %% 10 == 0)
#		print(i)
#	NJ_from_cognacy_data(dict.omission.create.pseudo.dataset(data, p.qu.to.1, p.0.to.1))
#	})
#class(dict.omission.pseudodatasets.NJ) = "multiPhylo"
#plotBS(data.NJ, dict.omission.pseudodatasets.NJ, type="phylogram", cex=1)
#title("p(?->1) = 0.13  p(0->1) = 0.03")
